@@ -62,19 +62,18 @@ SAMPLE OUTPUT
 {'full_text': 'Mail: 36', 'color': '#00FF00'}
 """
 import imaplib
-import os
 from threading import Thread
 from time import sleep
 from ssl import create_default_context
 from socket import setdefaulttimeout, error as socket_error
+from pathlib import Path
 
 STRING_UNAVAILABLE = "N/A"
 NO_DATA_YET = -1
 
 
 class Py3status:
-    """
-    """
+    """"""
 
     # available configuration parameters
     allow_urgent = False
@@ -124,8 +123,8 @@ class Py3status:
         self.idle_thread = Thread()
 
         if self.client_secret:
-            self.client_secret = os.path.expanduser(self.client_secret)
-        self.auth_token = os.path.expanduser(self.auth_token)
+            self.client_secret = Path(self.client_secret).expanduser()
+        self.auth_token = Path(self.auth_token).expanduser()
 
         if self.security not in ["ssl", "starttls"]:
             raise ValueError("Unknown security protocol")
@@ -187,8 +186,8 @@ class Py3status:
         self.creds = None
 
         # Open pickle file with access and refresh tokens if it exists
-        if os.path.exists(self.auth_token):
-            with open(self.auth_token, "rb") as token:
+        if self.auth_token.exists():
+            with self.auth_token.open("rb") as token:
                 self.creds = pickle.load(token)
 
         if not self.creds or not self.creds.valid:
@@ -203,7 +202,7 @@ class Py3status:
                     )
                     self.creds = flow.run_local_server(port=0)
                 # Save the credentials for the next run
-                with open(self.auth_token, "wb") as token:
+                with self.auth_token.open("wb") as token:
                     pickle.dump(self.creds, token)
             except TransportError as e:
                 # Treat the same as a socket_error
@@ -271,9 +270,7 @@ class Py3status:
             # Dovecot will responde with "+ idling", courier will return "+ entering idle mode"
             # RFC 2177 (https://tools.ietf.org/html/rfc2177) only requires the "+" character.
             if not response.lower().startswith("+"):
-                raise imaplib.IMAP4.abort(
-                    "While initializing IDLE: {}".format(response)
-                )
+                raise imaplib.IMAP4.abort(f"While initializing IDLE: {response}")
 
             # wait for changes (EXISTS, EXPUNGE, etc.):
             socket.settimeout(self.cache_timeout)
@@ -359,8 +356,7 @@ class Py3status:
 
                 if self.debug:
                     self.py3.log(
-                        "{} error - {}".format(error_type, e),
-                        level=self.py3.LOG_WARNING,
+                        f"{error_type} error - {e}", level=self.py3.LOG_WARNING,
                     )
                 self._disconnect()
 
@@ -368,13 +364,13 @@ class Py3status:
                 if retry_counter <= retry_max:
                     if self.debug:
                         self.py3.log(
-                            "Retrying ({}/{})".format(retry_counter, retry_max),
+                            f"Retrying ({retry_counter}/{retry_max})",
                             level=self.py3.LOG_INFO,
                         )
                     continue
                 break
             except (imaplib.IMAP4.error, Exception) as e:
-                self.mail_error = "Fatal error - {}".format(e)
+                self.mail_error = f"Fatal error - {e}"
                 self._disconnect()
                 self.mail_count = None
 
