@@ -98,7 +98,7 @@ Configuration parameters:
     format_wind: Formatting for wind degree and speed
         The 'gust' option represents the speed of wind gusts in the wind unit.
         Available placeholders:
-            icon, degree, speed, gust
+            icon, degree, speed, gust, direction
         (default '[\?if=speed {icon} {speed:.0f} {unit}]')
     icon_atmosphere: Icon for atmospheric conditions, like fog, smog, etc.
         (default '🌫')
@@ -169,11 +169,11 @@ Configuration parameters:
         case, such as 'Ft', and still be considered valid as long as it is in
         the below options.
         Options:
-            cm, ft, in, mm, m, yd
+            mm, cm, in
         (default 'in')
     unit_snow: Unit for snow fall
         Options:
-            cm, ft, in, mm, m, yd
+            mm, cm, in
         (default 'in')
     unit_temperature: Unit for temperature
         Options:
@@ -181,7 +181,7 @@ Configuration parameters:
         (default 'F')
     unit_wind: Unit for wind speed
         Options:
-            fsec, msec, mph, kmh
+            fsec, msec, mph, kmh, knot
         (default 'mph')
 
 Format placeholders:
@@ -206,6 +206,7 @@ Format placeholders:
         {unit} The unit specified
     format_wind:
         {degree} Current wind heading
+        {direction} Letter reprents direction e.g. N,NE,E etc
         {gust} Wind gusts speed in the specified unit
         {speed} Wind speed
         {unit} The unit specified
@@ -265,6 +266,7 @@ import datetime
 # API information
 OWM_CURR_ENDPOINT = "https://api.openweathermap.org/data/2.5/weather?"
 OWM_FUTURE_ENDPOINT = "https://api.openweathermap.org/data/2.5/forecast?"
+OWM_ONECALL_ENDPOINT = "https://api.openweathermap.org/data/2.5/onecall?"
 IP_ENDPOINT = "http://geo.ultrabug.fr"
 
 # Paths of information to extract from JSON
@@ -275,27 +277,41 @@ IP_LON = "//longitude"
 OWM_CLOUD_COVER = "//clouds/all"
 OWM_DESC = "//weather:0/main"
 OWM_DESC_LONG = "//weather:0/description"
-OWM_HUMIDITY = "//main/humidity"
-OWM_PRESSURE = "//main"
-OWM_RAIN = "//rain/3h"
-OWM_SNOW = "//snow/3h"
+OWM_HUMIDITY = "//humidity"
+OWM_PRESSURE = "//pressure"
+OWM_RAIN = "//rain/1h"
+OWM_SNOW = "//snow/1h"
 OWM_SUNRISE = "//sys/sunrise"
 OWM_SUNSET = "//sys/sunset"
-OWM_TEMP = "//main"
+OWM_TEMP_DAY = "//temp/day"
+OWM_TEMP_MAX = "//temp/max"
+OWM_TEMP_MIN = "//temp/min"
 OWM_WEATHER_ICON = "//weather:0/id"
-OWM_WIND = "//wind"
+OWM_WIND_SPEED = "//wind_speed"
+OWM_WIND_GUST = "//wind_gust"
+OWM_WIND_DEG = "//wind_deg"
+
+OWN_CURRENT_WIND_DEG = "//wind/deg"
+OWN_CURRENT_WIND_GUST = "//wind/gust"
+OWN_CURRENT_WIND_SPEED = "//wind/speed"
+OWM_CURRENT_TEMP = "//main/temp"
+OWM_CURRENT_TEMP_MAX = "//main/temp_max"
+OWM_CURRENT_TEMP_MIN = "//main/temp_min"
+OWM_CURRENT_HUMIDITY = "//main/humidity"
+OWM_CURRENT_PRESSURE = "//main/pressure"
 
 # Units constants
-RAIN_UNITS = {"cm", "ft", "in", "mm", "m", "yd"}
+RAIN_UNITS = {"mm", "cm", "in"}
 SNOW_UNITS = RAIN_UNITS
 TEMP_UNITS = {"c", "f", "k"}
-WIND_UNITS = {"fsec", "msec", "mph", "kmh"}
+WIND_UNITS = {"fsec", "msec", "mph", "kmh", "knot"}
 
 # Conversion factors
 FT_FROM_METER = 3.28084
 IN_FROM_MM = 0.0393701
 KMH_FROM_MSEC = 0.277778
 MPH_FROM_MSEC = 2.23694
+KNOT_FROM_MSEC = 1.94384
 
 # Thresholds options
 THRESHOLDS_ALL = "all"
@@ -362,6 +378,27 @@ class Py3status:
             ],
         }
 
+        update_config = {
+            "update_placeholder_format": [
+                {
+                    "placeholder_formats": {"amount": ":.2f"},
+                    "format_strings": ["format_rain", "format_snow"],
+                },
+                {
+                    "placeholder_formats": {
+                        "max": ":.1f",
+                        "min": ":.1f",
+                        "current": ":.1f",
+                    },
+                    "format_strings": ["format_temperature"],
+                },
+                {
+                    "placeholder_formats": {"speed": ":.1f", "gust": ":.1f"},
+                    "format_strings": ["format_wind"],
+                },
+            ],
+        }
+
     def _get_icons(self):
         if self.icons is None:
             self.icons = {}
@@ -385,15 +422,15 @@ class Py3status:
         for source in (defaults, self.icons):
             for key in source:
                 if not key.replace("_", "").isdigit():
-                    raise Exception("Invalid icon id: (%s)" % key)
+                    raise Exception(f"Invalid icon id: ({key})")
 
                 if "_" in key:
                     if key.count("_") != 1:
-                        raise Exception("Invalid icon range: %s" % key)
+                        raise Exception("fInvalid icon range: {key}")
 
                     # Populate each code
-                    (start, end) = tuple(map(int, key.split("_")))
-                    for code in range(start, end + 1):
+                    start, _, end = key.partition("_")
+                    for code in range(int(start), int(end) + 1):
                         data[code] = source[key]
 
                 else:
@@ -424,7 +461,7 @@ class Py3status:
             raise Exception("unit_wind is not recognized")
 
         # Check thresholds for validity
-        if set(self.thresholds.keys()) > THRESHOLDS_NAMES:
+        if set(self.thresholds) > THRESHOLDS_NAMES:
             raise Exception("threshold name(s) are not recognized")
 
         # Copy thresholds if available
@@ -477,7 +514,7 @@ class Py3status:
 
     def _get_loc_tz_info(self):
         # Preference a user-set location
-        if all(map(lambda x: x is not None, (self.location, self.city, self.country))):
+        if all(x is not None for x in (self.location, self.city, self.country)):
             return (self.location, self.city, self.country)
 
         data = {}
@@ -511,9 +548,9 @@ class Py3status:
 
         return (lat_lng, city, country)
 
-    def _get_weather(self, extras):
+    def _get_current_weather(self, extras):
         # Get and process the current weather
-        params = {"APPID": self.api_key, "lang": self.lang}
+        params = {"appid": self.api_key, "lang": self.lang}
         extras.update(params)
         return self._make_req(OWM_CURR_ENDPOINT, extras)
 
@@ -523,7 +560,7 @@ class Py3status:
             return []
         # Get raw data
         params = {
-            "APPID": self.api_key,
+            "appid": self.api_key,
             "lang": self.lang,
             "cnt": self.forecast_days + 1,
         }
@@ -531,7 +568,13 @@ class Py3status:
         data = self._make_req(OWM_FUTURE_ENDPOINT, extras)
         # Extract forecast
         weathers = data["list"]
-        return weathers[:-1] if (self.forecast_include_today) else weathers[1:]
+        return weathers[:-1] if self.forecast_include_today else weathers[1:]
+
+    def _get_onecall(self, extras):
+        # Get and process the current weather
+        params = {"appid": self.api_key, "lang": self.lang}
+        extras.update(params)
+        return self._make_req(OWM_ONECALL_ENDPOINT, extras)
 
     def _get_icon(self, wthr):
         # Lookup the icon from the weather code (default sunny)
@@ -555,12 +598,9 @@ class Py3status:
         inches = rain * IN_FROM_MM
 
         options = {
-            "mm": round(rain),
-            "cm": round(rain / 10),
-            "m": round(rain / 100),
-            "in": round(inches),
-            "ft": round(inches / 12),
-            "yd": round(inches / 36),
+            "mm": rain,
+            "cm": rain / 10,
+            "in": inches,
         }
 
         # Format the rain fall
@@ -581,12 +621,9 @@ class Py3status:
         inches = snow * IN_FROM_MM
 
         options = {
-            "mm": round(snow),
-            "cm": round(snow / 10),
-            "m": round(snow / 100),
-            "in": round(inches),
-            "ft": round(inches / 12),
-            "yd": round(inches / 36),
+            "mm": snow,
+            "cm": snow / 10,
+            "in": inches,
         }
 
         # Format the snow fall
@@ -600,7 +637,17 @@ class Py3status:
         )
 
     def _format_wind(self, wthr):
-        wind = self._jpath(wthr, OWM_WIND, dict())
+        wind = {
+            "deg": self._jpath(
+                wthr, OWM_WIND_DEG, self._jpath(wthr, OWN_CURRENT_WIND_DEG, 0)
+            ),
+            "gust": self._jpath(
+                wthr, OWM_WIND_GUST, self._jpath(wthr, OWN_CURRENT_WIND_GUST, 0)
+            ),
+            "speed": self._jpath(
+                wthr, OWM_WIND_SPEED, self._jpath(wthr, OWN_CURRENT_WIND_SPEED, 0)
+            ),
+        }
 
         # Speed and Gust
         msec_speed = wind["speed"] if ("speed" in wind) else 0
@@ -620,12 +667,33 @@ class Py3status:
                 "speed": msec_speed * KMH_FROM_MSEC,
                 "gust": msec_gust * KMH_FROM_MSEC,
             },
+            "knot": {
+                "speed": msec_speed * KNOT_FROM_MSEC,
+                "gust": msec_gust * KNOT_FROM_MSEC,
+            },
         }
 
         # Get the choice and add more
         choice = options[self.unit_wind.lower()]
         choice["icon"] = self.icon_wind
         choice["degree"] = wind["deg"] if ("deg" in wind) else 0
+        if wind["deg"] >= 20 and wind["deg"] < 65:
+            direction = "NE"
+        elif wind["deg"] >= 65 and wind["deg"] < 110:
+            direction = "E"
+        elif wind["deg"] >= 110 and wind["deg"] < 155:
+            direction = "SE"
+        elif wind["deg"] >= 155 and wind["deg"] < 200:
+            direction = "S"
+        elif wind["deg"] >= 200 and wind["deg"] < 245:
+            direction = "SW"
+        elif wind["deg"] >= 245 and wind["deg"] < 290:
+            direction = "W"
+        elif wind["deg"] >= 290 and wind["deg"] < 335:
+            direction = "NW"
+        else:
+            direction = "N"
+        choice["direction"] = direction
         choice["unit"] = self.unit_wind
 
         # Format the wind speed
@@ -635,7 +703,9 @@ class Py3status:
         # Format the humidity (default zero humidity)
         humidity_data = {
             "icon": self.icon_humidity,
-            "humidity": self._jpath(wthr, OWM_HUMIDITY, 0),
+            "humidity": self._jpath(
+                wthr, OWM_HUMIDITY, self._jpath(wthr, OWM_CURRENT_HUMIDITY, 0)
+            ),
         }
 
         for x in self.thresholds_init["format_humidity"]:
@@ -646,38 +716,52 @@ class Py3status:
 
     def _format_pressure(self, wthr):
         # Get data and add the icon
-        pressure = self._jpath(wthr, OWM_PRESSURE, dict())
-        pressure["icon"] = self.icon_pressure
+        pressure = {
+            "icon": self.icon_pressure,
+            "pressure": self._jpath(
+                wthr, OWM_PRESSURE, self._jpath(wthr, OWM_CURRENT_PRESSURE, 0)
+            ),
+        }
 
         # Format the barometric pressure
         return self.py3.safe_format(self.format_pressure, pressure)
 
     def _format_temp(self, wthr):
         # Get Kelvin data (default absolute zero)
-        kelvin = self._jpath(wthr, OWM_TEMP, 0)
+        kelvin = {
+            "day": self._jpath(
+                wthr, OWM_TEMP_DAY, self._jpath(wthr, OWM_CURRENT_TEMP, "")
+            ),
+            "max": self._jpath(
+                wthr, OWM_TEMP_MAX, self._jpath(wthr, OWM_CURRENT_TEMP_MAX, "")
+            ),
+            "min": self._jpath(
+                wthr, OWM_TEMP_MIN, self._jpath(wthr, OWM_CURRENT_TEMP_MIN, "")
+            ),
+        }
 
         # Temperature conversion methods
         def kToC(val):
             return val - 273.15
 
         def kToF(val):
-            return val * (9.0 / 5.0) - 459.67
+            return val * 9 / 5 - 459.67
 
         options = {
             "c": {
-                "current": round(kToC(kelvin["temp"])),
-                "max": round(kToC(kelvin["temp_max"])),
-                "min": round(kToC(kelvin["temp_min"])),
+                "current": kToC(kelvin["day"]),
+                "max": kToC(kelvin["max"]),
+                "min": kToC(kelvin["min"]),
             },
             "f": {
-                "current": round(kToF(kelvin["temp"])),
-                "max": round(kToF(kelvin["temp_max"])),
-                "min": round(kToF(kelvin["temp_min"])),
+                "current": kToF(kelvin["day"]),
+                "max": kToF(kelvin["max"]),
+                "min": kToF(kelvin["min"]),
             },
             "k": {
-                "current": round(kelvin["temp"]),
-                "max": round(kelvin["temp_max"]),
-                "min": round(kelvin["temp_min"]),
+                "current": kelvin["day"],
+                "max": kelvin["max"],
+                "min": kelvin["min"],
             },
         }
 
@@ -697,7 +781,7 @@ class Py3status:
 
     def _format_sunrise(self, wthr):
         # Get the time for sunrise (default is the start of time)
-        dt = datetime.datetime.utcfromtimestamp(self._jpath(wthr, OWM_SUNRISE, 0))
+        dt = datetime.datetime.fromtimestamp(self._jpath(wthr, OWM_SUNRISE, 0))
 
         # Format the sunrise
         replaced = dt.strftime(self.format_sunrise)
@@ -705,7 +789,7 @@ class Py3status:
 
     def _format_sunset(self, wthr):
         # Get the time for sunset (default is the start of time)
-        dt = datetime.datetime.utcfromtimestamp(self._jpath(wthr, OWM_SUNSET, 0))
+        dt = datetime.datetime.fromtimestamp(self._jpath(wthr, OWM_SUNSET, 0))
 
         # Format the sunset
         replaced = dt.strftime(self.format_sunset)
@@ -734,9 +818,9 @@ class Py3status:
 
         return data
 
-    def _format(self, wthr, fcsts, city, country):
+    def _format(self, current_wthr, fcsts, city, country):
         # Format all sections
-        today = self._format_dict(wthr, city, country)
+        today = self._format_dict(current_wthr, city, country)
 
         # Insert forecasts
         forecasts = []
@@ -759,20 +843,32 @@ class Py3status:
         if loc_tz_info is not None:
             (coords, city, country) = loc_tz_info
             if coords:
-                extras = {"lat": coords[0], "lon": coords[1]}
+                current_api_params = {"lat": coords[0], "lon": coords[1]}
             elif city:
-                extras = {"q": city}
-            wthr = self._get_weather(extras)
-            fcsts = self._get_forecast(extras)
+                current_api_params = {"q": city}
 
             # try to get a nice city name
-            city = wthr.get("name") or city or "unknown"
+            current_wthr = self._get_current_weather(current_api_params)
+            owm_city = current_wthr.get("name") or city or "unknown"
             # get the best country we can
             if not country:
-                sys = wthr.get("sys", {})
+                sys = current_wthr.get("sys", {})
                 country = sys.get("country", "unknown")
+            try:
+                lat = current_wthr["coord"]["lat"]
+                lon = current_wthr["coord"]["lon"]
+            except Exception:
+                raise Exception("no latitude/longitude found for your config")
 
-            text = self._format(wthr, fcsts, city, country)
+            # onecall = forecasts
+            onecall_api_params = {"lat": lat, "lon": lon}
+            onecall = self._get_onecall(onecall_api_params)
+            onecall_daily = onecall["daily"]
+
+            fcsts_days = self.forecast_days + 1
+            text = self._format(
+                current_wthr, onecall_daily[1:fcsts_days], owm_city, country,
+            )
 
         return {
             "full_text": text,
@@ -784,49 +880,42 @@ if __name__ == "__main__":
     """
     Run module in test mode.
     """
-    import os
+    from os import getenv
     from py3status.module_test import module_test
 
-    # All possible outputs
-    all_string = "/".join(
-        [
-            "{clouds}",
-            "{description}",
-            "{main}",
-            "{humidity}",
-            "{pressure}",
-            "{snow}",
-            "{sunrise}",
-            "{sunset}",
-            "{temperature}",
-            "{wind}",
-        ]
-    )
+    def colorize(names, color="lightgreen", separator=None):
+        if separator is None:
+            separator = "[\?color={c}&show  / ]".format(c=color)
+        body = "[\?color={c}&show {n}] [\?color={n} {{{n}}}]"
+        return separator.join([body.format(c=color, n=name) for name in names])
 
-    module_test(
-        Py3status,
-        config={
-            "api_key": os.getenv("OWM_API_KEY"),
-            # Select icons
-            "icons": {"200": "☔", "230_232": "🌧"},
-            # Complete configuration
-            "format_clouds": "{icon} {coverage}%",
-            "format_humidity": "{icon} {humidity}%",
-            "format_pressure": "{icon} {pressure} Pa, sea: {sea_level} Pa",
-            "format_rain": "{icon} {amount:.0f} in",
-            "format_snow": "{icon} {amount:.0f} in",
-            "format_temperature": (
-                r"{icon}: max: [\?color=max {max:.0f}°F], "
-                r"min: [\?color=min {min:.0f}°F], "
-                r"current: [\?color=current {current:.0f}°F]"
-            ),
-            "format_wind": (
-                "{icon} {degree}°, gust: {gust:.0f} mph, " "speed: {speed:.0f} mph"
-            ),
-            "format": ("{city}, {country}: {icon} " + all_string + "//{forecast}"),
-            "format_forecast": ("{icon} " + all_string),
-            # Miscellaneous
-            "forecast_days": 1,
-            "format_forecast_separator": "//",
-        },
-    )
+    # fmt: off
+    weather_placeholders = [
+        "icon", "clouds", "description", "humidity", "main", "pressure", "rain",
+        "snow", "sunrise", "sunset", "temperature", "wind",
+    ]
+    format_placeholders = ["city", "country"] + weather_placeholders + ["forecast"]
+    # fmt: on
+
+    config = {
+        # Miscellaneous
+        "api_key": getenv("OWM_API_KEY"),
+        "icons": {"200": "☔", "230_232": "🌧"},
+        "format_forecast_separator": "\?color=tomato  separator ",
+        "forecast_days": 1,
+        # Format
+        "format": colorize(format_placeholders, "lightblue"),
+        "format_forecast": colorize(weather_placeholders, "pink"),
+        # Weather
+        "format_clouds": colorize(["icon", "coverage"]),
+        "format_humidity": colorize(["icon", "humidity"]),
+        "format_pressure": colorize(["icon", "pressure", "sea_level"]),
+        "format_rain": colorize(["icon", "amount", "unit"]),
+        "format_snow": colorize(["icon", "amount", "unit"]),
+        "format_temperature": colorize(["icon", "max", "min", "current", "unit"]),
+        "format_wind": colorize(
+            ["icon", "degree", "direction", "gust", "speed", "unit"]
+        ),
+    }
+
+    module_test(Py3status, config)
